import edu.njit.decaf3.DECAF;
import edu.njit.decaf3.ThresholdAnalysis;
import edu.njit.decaf3.helper.Control;
import edu.njit.decaf3.helper.JSON;
import edu.njit.decaf3.structure.Simulation;

/**
 * DECAF - Dependability Evaluator of Cascading Failures Revision 3
 *
 * @author Sashank Tadepalli, Mihir Sanghavi
 *
 */
public class Program {
	public static void main(String[] args) {
		if(args.length < 2 )
			Control.end("Not enough arguments");

        try {
            Simulation sim = JSON.parse(args[1]);
            DECAF decafSim;
            ThresholdAnalysis ta;
            long trees, trans;
            double mttf, ssu, qst1, qst2, nft, fpt;

            if(args[0].equalsIgnoreCase("analyze")){
                ta = new ThresholdAnalysis(sim);
                ta.setPrintType(ThresholdAnalysis.PRETTY);
                ta.getOptimalRateThreshold();
                System.out.println("Total Compute Time: " + ta.AnalyzeTime + "\n" + ta);

            } else if(args[0].equalsIgnoreCase("run") || args[0].equalsIgnoreCase("runcsv")) {
                decafSim = new DECAF(sim);

	            if(args.length > 2)
                decafSim.heightThreshold = Integer.parseInt(args[2]);
	            if(args.length > 3)
                decafSim.rateThreshold = Double.parseDouble(args[3]);
	            if(args.length > 4)
                decafSim.treeThreshold = Integer.parseInt(args[4]);

                decafSim.run();

                mttf = decafSim.getMTTF();
                ssu = decafSim.getSSU();
                trans = decafSim.numberOfTransitions;
                trees = decafSim.numberOfTrees;
                qst1 = decafSim.MTTFSolveTime;
	            qst2 = decafSim.SSUSolveTime;
                nft = decafSim.nonFailureTime;
                fpt = decafSim.failureTransitionTime;

                if(args[0].equalsIgnoreCase("runcsv")){
                    System.out.println(decafSim.length + "," + decafSim.upLength + "," + trees + "," + trans +
                            "," + nft + "," + fpt + "," + qst1 + "," + qst2 + "," + mttf + "," + ssu);
                } else {
                    System.out.println("\n# States:\t" + decafSim.length);
                    System.out.println("# Up-states:\t" + decafSim.upLength);
                    System.out.println("# Trees:\t" + trees);
                    System.out.println("# Trans:\t" + trans + "\n");
                    System.out.println("MTTF:\t\t" + mttf);
                    System.out.println("SSU:\t\t" + ssu + "\n");
                    System.out.println("Time NFT:\t" + nft);
                    System.out.println("Time FT:\t" + fpt);
                    System.out.println("Time MTTF:\t" + qst1);
                    System.out.println("Time SSU:\t" + qst2);
                    System.out.println("Total Time:\t" + (nft + fpt + qst1 + qst2));
                }
            } else {
                Control.end("");
            }
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}